@@ -1,21 +1,17 @@
 # -*- coding: utf-8 -*-
-"""
-Created on Wed Oct 31 16:23:13 2018
-
-@author: samschott
-"""
 
 # system imports
 import sys
 import os
 import time
+from traceback import format_exception
 from subprocess import Popen
 from datetime import timedelta, datetime
 from shlex import quote
 
 # external packages
 import click
-from PyQt5 import QtCore, QtWidgets, QtGui
+from PyQt6 import QtCore, QtWidgets, QtGui
 
 # maestral modules
 from maestral.constants import (
@@ -34,18 +30,26 @@ from maestral.daemon import (
     Start,
     CommunicationError,
 )
-from maestral.errors import KeyringAccessError
+from maestral.exceptions import (
+    KeyringAccessError,
+    NoDropboxDirError,
+    TokenExpiredError,
+    TokenRevokedError,
+    MaestralApiError,
+    UpdateCheckError,
+    NotLinkedError,
+)
 
 # local imports
-from maestral_qt import __url__
-from maestral_qt.setup_dialog import SetupDialog
-from maestral_qt.relink_dialog import RelinkDialog
-from maestral_qt.dropbox_location_dialog import DropboxLocationDialog
-from maestral_qt.settings_window import SettingsWindow
-from maestral_qt.activity_window import ActivityWindow
-from maestral_qt.sync_issues_window import SyncIssueWindow
-from maestral_qt.resources import system_tray_icon, APP_ICON_PATH
-from maestral_qt.utils import (
+from . import __url__
+from .setup_dialog import SetupDialog
+from .relink_dialog import RelinkDialog
+from .dropbox_location_dialog import DropboxLocationDialog
+from .settings_window import SettingsWindow
+from .activity_window import ActivityWindow
+from .sync_issues_window import SyncIssueWindow
+from .resources import system_tray_icon, APP_ICON_PATH
+from .utils import (
     BackgroundTask,
     MaestralBackgroundTask,
     elide_string,
@@ -53,14 +57,14 @@ from maestral_qt.utils import (
     IS_MACOS,
     IS_BUNDLE,
 )
-from maestral_qt.widgets import (
+from .widgets import (
     BackgroundTaskProgressDialog,
     UserDialog,
     show_dialog,
     show_stacktrace_dialog,
     show_update_dialog,
 )
-from maestral_qt.autostart import AutoStart
+from .autostart import AutoStart
 
 
 # noinspection PyTypeChecker,PyArgumentList
@@ -114,6 +118,7 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
         self.show_when_systray_available()
 
         self.menu = QtWidgets.QMenu()
+        self.menu.setSeparatorsCollapsible(False)
         self.setContextMenu(self.menu)
 
         self.setup_ui_unlinked()
@@ -140,13 +145,11 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
             self._current_icon = icon_name
             super().setIcon(icon)
 
-    @QtCore.pyqtSlot()
     def _onContextMenuAboutToShow(self):
         self._context_menu_visible = True
         if self.loading_done:
             self.update_status()
 
-    @QtCore.pyqtSlot()
     def _onContextMenuAboutToHide(self):
         self._context_menu_visible = False
 
@@ -189,7 +192,9 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
             self.update_error()
             return
 
-        if pending_link:
+        pending_folder = self.mdbx.pending_dropbox_folder
+
+        if pending_link or pending_folder:
             self.loading_done = SetupDialog.configureMaestral(self.mdbx)
         else:
             self.loading_done = True
@@ -304,10 +309,10 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
         self.snooze30.triggered.connect(lambda: snooze_for(30))
         self.snooze60.triggered.connect(lambda: snooze_for(60))
         self.snooze480.triggered.connect(lambda: snooze_for(480))
-        self.snoozeSeparator = QtWidgets.QAction()
+        self.snoozeSeparator = QtGui.QAction()
         self.snoozeSeparator.setSeparator(True)
 
-        self.resumeNotificationsAction = QtWidgets.QAction("Turn on notifications")
+        self.resumeNotificationsAction = QtGui.QAction("Turn on notifications")
         self.resumeNotificationsAction.triggered.connect(lambda: snooze_for(0))
 
         self.syncIssuesAction = self.menu.addAction("Show Sync Issues...")
@@ -340,7 +345,6 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
 
     # callbacks for user interaction
 
-    @QtCore.pyqtSlot()
     def auto_check_for_updates(self):
 
         last_update_check = self.mdbx.get_state("app", "update_notification_last")
@@ -353,7 +357,6 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
             )
             checker.sig_result.connect(self._notify_updates_auto)
 
-    @QtCore.pyqtSlot()
     def on_check_for_updates_clicked(self):
 
         checker = MaestralBackgroundTask(
@@ -366,77 +369,71 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
         checker.sig_result.connect(self._progress_dialog.accept)
         checker.sig_result.connect(self._notify_updates_user_requested)
 
-    @QtCore.pyqtSlot(dict)
     def _notify_updates_user_requested(self, res):
 
-        if res["error"]:
-            show_dialog("Could not check for updates", res["error"], level="warning")
-        elif res["update_available"]:
+        if isinstance(res, UpdateCheckError):
+            show_dialog(res.title, res.message, level="warning")
+        elif res.update_available:
             show_update_dialog(res["latest_release"], res["release_notes"])
-        elif not res["update_available"]:
+        else:
             message = "Maestral v{} is the newest version available.".format(
-                res["latest_release"]
+                res.latest_release
             )
             show_dialog("You’re up-to-date!", message)
 
-    @QtCore.pyqtSlot(dict)
     def _notify_updates_auto(self, res):
 
-        if res["update_available"]:
+        if res.update_available:
             self.mdbx.set_state("app", "update_notification_last", time.time())
-            show_update_dialog(res["latest_release"], res["release_notes"])
+            show_update_dialog(res.latest_release, res.release_notes)
 
-    @QtCore.pyqtSlot()
     def on_website_clicked(self):
         """Open the Dropbox website."""
         click.launch("https://www.dropbox.com/")
 
-    @QtCore.pyqtSlot()
     def on_folder_clicked(self):
         """Open the Dropbox folder."""
         click.launch(self.mdbx.dropbox_path)
 
-    @QtCore.pyqtSlot()
     def on_help_clicked(self):
         """Open the Dropbox help website."""
         click.launch(f"{__url__}/docs")
 
-    @QtCore.pyqtSlot()
     def on_start_stop_clicked(self):
         """Pause / resume syncing on menu item clicked."""
-        if self.pauseAction.text() == self.PAUSE_TEXT:
-            self.mdbx.stop_sync()
-            self.pauseAction.setText(self.RESUME_TEXT)
-        elif self.pauseAction.text() == self.RESUME_TEXT:
-            self.mdbx.start_sync()
-            self.pauseAction.setText(self.PAUSE_TEXT)
-        elif self.pauseAction.text() == "Start Syncing":
-            self.mdbx.start_sync()
-            self.pauseAction.setText(self.PAUSE_TEXT)
 
-    @QtCore.pyqtSlot()
+        try:
+            if self.pauseAction.text() == self.PAUSE_TEXT:
+                self.mdbx.stop_sync()
+                self.pauseAction.setText(self.RESUME_TEXT)
+            elif self.pauseAction.text() == self.RESUME_TEXT:
+                self.mdbx.start_sync()
+                self.pauseAction.setText(self.PAUSE_TEXT)
+            elif self.pauseAction.text() == "Start Syncing":
+                self.mdbx.start_sync()
+                self.pauseAction.setText(self.PAUSE_TEXT)
+        except (NotLinkedError, NoDropboxDirError):
+            self.restart()
+
     def on_settings_clicked(self):
         self.settings_window.show()
         self.settings_window.raise_()
         self.settings_window.activateWindow()
 
-    @QtCore.pyqtSlot()
     def on_sync_issues_clicked(self):
         self.sync_issues_window = SyncIssueWindow(self.mdbx)
         self.sync_issues_window.show()
         self.sync_issues_window.raise_()
         self.sync_issues_window.activateWindow()
-        self.sync_issues_window.setAttribute(QtCore.Qt.WA_DeleteOnClose)
+        self.sync_issues_window.setAttribute(QtCore.Qt.WidgetAttribute.WA_DeleteOnClose)
 
-    @QtCore.pyqtSlot()
     def on_activity_clicked(self):
         self.activity_window = ActivityWindow(self.mdbx)
         self.activity_window.show()
         self.activity_window.raise_()
         self.activity_window.activateWindow()
-        self.activity_window.setAttribute(QtCore.Qt.WA_DeleteOnClose)
+        self.activity_window.setAttribute(QtCore.Qt.WidgetAttribute.WA_DeleteOnClose)
 
-    @QtCore.pyqtSlot()
     def on_rebuild_clicked(self):
         self.rebuild_dialog = UserDialog(
             title="Rebuilt Maestral's sync index?",
@@ -448,13 +445,12 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
             ),
             button_names=("Rebuild", "Cancel"),
         )
-        res = self.rebuild_dialog.exec_()
-        if res == UserDialog.Accepted:
+        res = self.rebuild_dialog.exec()
+        if res == UserDialog.DialogCode.Accepted:
             self.mdbx.rebuild_index()
 
     # callbacks to update GUI
 
-    @QtCore.pyqtSlot()
     def update_snoozed(self):
         minutes = self.mdbx.notification_snooze
 
@@ -477,7 +473,6 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
         n_sync_errors = len(self.mdbx.sync_errors)
         status = self.mdbx.status
         is_paused = self.mdbx.paused
-        is_connected = self.mdbx.connected
 
         # update icon
         if n_sync_errors > 0 and status == IDLE:
@@ -527,16 +522,16 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
 
         err = errs[-1]
 
-        if err["type"] == "NoDropboxDirError":
+        if isinstance(err, NoDropboxDirError):
             # Show location dialog dialog.
             self._dbx_location_dialog = DropboxLocationDialog(self.mdbx)
             self._dbx_location_dialog.show()
             self._dbx_location_dialog.raise_()
 
-        elif err["type"] in ("TokenRevokedError", "TokenExpiredError"):
+        elif isinstance(err, (TokenRevokedError, TokenExpiredError)):
             # Show relink dialog.
 
-            if err["type"] == "TokenRevokedError":
+            if isinstance(err, TokenExpiredError):
                 reason = RelinkDialog.REVOKED
             else:
                 reason = RelinkDialog.EXPIRED
@@ -545,21 +540,27 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
             self._relink_dialog.show()
             self._relink_dialog.raise_()
 
-        elif "MaestralApiError" in err["inherits"] or "SyncError" in err["inherits"]:
+        elif isinstance(err, MaestralApiError):
             # This is a known error. We show the error message and the corresponding
             # file path, if any.
 
-            message = markup_urls(err["message"])
-            filename = err["dbx_path"] or err["local_path"]
+            message = markup_urls(err.message)
+            filename = err.dbx_path or err.local_path
 
             if filename:
                 message = f"Path: {filename}\n{message}"
 
-            show_dialog(err["title"], message, level="error")
+            show_dialog(err.title, message, level="error")
 
         else:
-            # This is an unexpected error. We show the full stacktrace.
-            show_stacktrace_dialog(err["traceback"])
+            # This is an unexpected error. We show the full stacktrace if available.
+            if err.__traceback__:
+                details = "".join(
+                    format_exception(err.__class__, err, err.__traceback__)
+                )
+                show_stacktrace_dialog(details)
+            else:
+                show_dialog("An unexpected error occurred", str(err), level="error")
 
     def contextMenuVisible(self):
         return self._context_menu_visible
@@ -611,14 +612,11 @@ class MaestralGuiApp(QtWidgets.QSystemTrayIcon):
 # noinspection PyArgumentList
 def run(config_name="maestral"):
     """
-    This is the main interactive entry point which starts the PyQt5 GUI.
+    This is the main interactive entry point which starts the Qt GUI.
 
     :param str config_name: Name of Maestral config to run.
     """
 
-    QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
-    QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
-
     app = QtWidgets.QApplication(["Maestral"])
     app.setWindowIcon(QtGui.QIcon(APP_ICON_PATH))
     app.setQuitOnLastWindowClosed(False)
